package examples.hibernate;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.Transaction;
import org.hibernate.criterion.Order;

import java.util.List;


import examples.hibernate.domain.*;

public class Exercise {
    private SessionFactory factory;
    
    public Exercise(SessionFactory factory) {
	this.factory = factory;
    }
        
    /**
     * Runs an operation within a unit of work
     * 
     * @param operation
     */
    public void runOperation(String operation) {
	Session session = factory.getCurrentSession();  // obtain/start unit of work
	Transaction tx = null;
	try {
	    tx = session.beginTransaction();            // start transaction
	    // do some work
	    if ("Create".equals(operation)) {
		System.out.println("Creating initial data set.");
		Airline airline = new Airline("Air Persistence");
		Airplane airplane = new Airplane("Air Force One", 21000, 60, 71, "General Electric");  // creates an Engine internally
		airline.addAirplane(airplane);
		Passenger passenger = new Passenger("Mr. President", "1");
		airplane.addPassenger(passenger);
		System.out.println(airline);		
	    } else if ("Read".equals(operation)) {
		System.out.println("Reading data set.");
		List<Airline> airlines = (List<Airline>) session.createCriteria(Airline.class)
						.addOrder( Order.asc("name") )
		                                .list();
		for (Airline airline : airlines) {
		    System.out.println(airline);
		}
	    } else if ("Modify".equals(operation)) {
		System.out.println("Modifying all passengers from last Airline.");
		Airline airline = (Airline) session.createCriteria(Airline.class)
						.addOrder( Order.desc("name") )
						.list().get(0);
		for (Airplane airplane : airline.getAirplanes()) {
		    for (Passenger passenger : airplane.getPassengers()) {
			passenger.setName(passenger.getName() + " (modified)");
		    }
		}
	    } else if ("Delete".equals(operation)) {
		System.out.println("Deleting first Airline.");
		Airline airline = (Airline) session.createCriteria(Airline.class)
						.addOrder( Order.asc("name") )
		                                .list().get(0);

		// Must remove airplane associations with other objects. Orphaned objects will be deleted by the GC
		for (Airplane airplane : airline.getAirplanes()) {
		    for (Passenger passenger : airplane.getPassengers()) {
			airplane.removePassenger(passenger);
		    }
		}
	    } else {
		System.out.println("Sintax: Exercise <operation>");
		System.out.println("\t<operation> can be one of:");
		System.out.println("\t\tCreate\tCreates the initial data set. Requires an empty database");
		System.out.println("\t\tRead\tReads and prints the data set. Requires a previously populated database");
		System.out.println("\t\tModify\tReads, modifies and prints the data set. Requires a previously populated database");
		System.out.println("\t\tDelete\tDeletes the data set. Requires a previously populated database");
	    }
	    tx.commit();                                // commit transaction & end unit of work
	} catch (RuntimeException ex) {
	    if (tx != null) tx.rollback();              // abort transaction
	    throw ex;
	}
	
    }
    
    public Runnable parseOperation(String operation) throws Exception {
	return (Runnable) Class.forName(operation + "Operation").newInstance();
    }
    
    // main application loop
    public static void main(String args[]) throws Exception {
	Exercise app = new Exercise(PersistenceUtil.getSessionFactory());
	String operation = null;
	if (args.length == 1) {
	    operation = args[0];
	}
	
	app.runOperation(operation);
    }
}
